package com.steerinfo.dil.controller;

import com.steerinfo.dil.feign.TmstruckFeign;
import com.steerinfo.dil.service.ITmstruckWeightResultService;
import com.steerinfo.dil.service.impl.TmstruckWeightResultServiceImpl;
import com.steerinfo.dil.util.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:HuJianGuo
 * @GreateTime:2021/10/25 17:34
 * @Version:V2.0
 */
@RestController
@RequestMapping("/${api.version}/tmstruckweightresult")
public class TmstruckWeightResultController extends BaseRESTfulController {

    @Autowired
    TmstruckWeightResultServiceImpl tmstruckWeightResultServiceImpl;

    @Autowired
    TmstruckFeign tmstruckFeign;


    /**
     * 接收计量实绩(计量实绩)
     * @param mapList
     * @return
     */
    @ApiModelProperty(value = "接收计量实绩")
    @ApiOperation(value = "接收计量实绩")
    @ApiParam(name = "orderNumber")

    @ApiImplicitParams({
            @ApiImplicitParam(
                    name = "orderNumber", value = "运输订单号", required = true, dataType = "String"
            ),
            @ApiImplicitParam(
                    name = "resultCalculateNumber", value = "计皮/计毛汽车衡编号", required = true, dataType = "String"
            ),
            @ApiImplicitParam(
                    name = "resultWeight", value = "皮重或者毛重", required = true, dataType = "Double"
            ),
            @ApiImplicitParam(
                    name = "resultWeightTime", value = "计皮/计毛时间", required = true, dataType = "String"
            ),
            @ApiImplicitParam(
                    name = "resultNetWeight", value = "净重", required = true, dataType = "Double"
            ), @ApiImplicitParam(
            name = "resultPoundNo", value = "磅单号", required = true, dataType = "String"
    ), @ApiImplicitParam(
            name = "weightType", value = "计皮/计毛", required = true, dataType = "String"
    )
    }
    )
    @PostMapping("/receiveTmsTruckWeightResult")
    public RESTfulResult receiveTmsTruckWeightResult(@RequestBody(required = false) List<Map<String, Object>> mapList) {
        String s = tmstruckWeightResultServiceImpl.receiveTmsTruckWeightResult(mapList);
        return success(s);
    }


    /**
     * 计量退货异常
     * @param orderNo
     * @return
     */
    @PostMapping("/abnormalReturn")
    @ApiOperation("value=触发退货实绩")
    @ApiImplicitParam(name = "orderNo",value = "运输订单号",required = true,dataType = "String")
    public RESTfulResult sendAbnormalReturn(String orderNo){
        Map<String, Object> map = new HashMap<>();
        map.put("purchOrder",orderNo);
        return success(tmstruckFeign.sendAbnormalReturn(map));
    }


}
